@@ -66,6 +66,7 @@ from .smsgateway import smsgateway_blueprint
 from .clienttype import client_blueprint
 from .subscriptions import subscriptions_blueprint
 from .monitoring import monitoring_blueprint
+from .tokengroup import tokengroup_blueprint
 from privacyidea.api.lib.postpolicy import postrequest, sign_response
 from ..lib.error import (privacyIDEAError,
                          AuthError, UserError,
@@ -73,6 +74,7 @@ from ..lib.error import (privacyIDEAError,
 from privacyidea.lib.utils import get_client_ip
 from privacyidea.lib.user import User
 import datetime
+import threading
 
 log = logging.getLogger(__name__)
 
@@ -137,6 +139,7 @@ def before_userendpoint_request():
 @client_blueprint.before_request
 @subscriptions_blueprint.before_request
 @monitoring_blueprint.before_request
+@tokengroup_blueprint.before_request
 @admin_required
 def before_admin_request():
     before_request()
@@ -221,6 +224,7 @@ def before_request():
                         "privacyidea_server": privacyidea_server,
                         "action": "{0!s} {1!s}".format(request.method, request.url_rule),
                         "action_detail": "",
+                        "thread_id": u"{0!s}".format(threading.current_thread().ident),
                         "info": ""})
 
     if g.logged_in_user.get("role") == "admin":
@@ -258,6 +262,7 @@ def before_request():
 @validate_blueprint.after_request
 @register_blueprint.after_request
 @recover_blueprint.after_request
+@tokengroup_blueprint.after_request
 @jwtauth.after_request
 @postrequest(sign_response, request=request)
 def after_request(response):
@@ -283,6 +288,7 @@ def after_request(response):
 @eventhandling_blueprint.app_errorhandler(AuthError)
 @subscriptions_blueprint.app_errorhandler(AuthError)
 @monitoring_blueprint.app_errorhandler(AuthError)
+@tokengroup_blueprint.app_errorhandler(AuthError)
 def auth_error(error):
     if "audit_object" in g:
         message = ''
@@ -317,6 +323,7 @@ def auth_error(error):
 @subscriptions_blueprint.app_errorhandler(PolicyError)
 @monitoring_blueprint.app_errorhandler(PolicyError)
 @ttype_blueprint.app_errorhandler(PolicyError)
+@tokengroup_blueprint.app_errorhandler(PolicyError)
 def policy_error(error):
     if "audit_object" in g:
         g.audit_object.add_to_log({"info": error.message}, add_with_comma=True)
@@ -338,6 +345,7 @@ def policy_error(error):
 @recover_blueprint.app_errorhandler(ResourceNotFoundError)
 @subscriptions_blueprint.app_errorhandler(ResourceNotFoundError)
 @ttype_blueprint.app_errorhandler(ResourceNotFoundError)
+@tokengroup_blueprint.errorhandler(ResourceNotFoundError)
 def resource_not_found_error(error):
     """
     This function is called when an ResourceNotFoundError occurs.
@@ -364,6 +372,7 @@ def resource_not_found_error(error):
 @subscriptions_blueprint.app_errorhandler(privacyIDEAError)
 @monitoring_blueprint.app_errorhandler(privacyIDEAError)
 @ttype_blueprint.app_errorhandler(privacyIDEAError)
+@tokengroup_blueprint.app_errorhandler(privacyIDEAError)
 def privacyidea_error(error):
     """
     This function is called when an privacyIDEAError occurs.
@@ -391,6 +400,7 @@ def privacyidea_error(error):
 @subscriptions_blueprint.app_errorhandler(500)
 @monitoring_blueprint.app_errorhandler(500)
 @ttype_blueprint.app_errorhandler(500)
+@tokengroup_blueprint.app_errorhandler(500)
 def internal_error(error):
     """
     This function is called when an internal error (500) occurs.
