@@ -254,7 +254,7 @@ def get_all_params(request):
         # Add the unquoted HTML and form parameters
         return_param = {key: unquote(value) for (key, value) in param.items()}
 
-    if request.json:
+    if request.is_json:
         log.debug(u"Update params in request {0!s} {1!s} with JSON data.".format(request.method,
                                                                                  request.base_url))
         # Add the original JSON data
@@ -400,7 +400,6 @@ def attestation_certificate_allowed(cert_info, allowed_certs_pols):
     if not cert_info:
         return not allowed_certs_pols
 
-
     if allowed_certs_pols:
         for allowed_cert in allowed_certs_pols:
             tag, matching, _rest = allowed_cert.split("/", 3)
@@ -412,6 +411,7 @@ def attestation_certificate_allowed(cert_info, allowed_certs_pols):
 
     return True
 
+
 def is_fqdn(x):
     """
     Check whether a given string could plausibly be a FQDN.
