@@ -39,6 +39,7 @@ class ERROR:
     TOKENADMIN = 301
     CONFIGADMIN = 302
     POLICY = 303
+    IMPORTADMIN = 304
     VALIDATE = 401
     REGISTRATION = 402
     AUTHENTICATE = 403
@@ -48,7 +49,10 @@ class ERROR:
     AUTHENTICATE_DECODING_ERROR = 4304
     AUTHENTICATE_TOKEN_EXPIRED = 4305
     AUTHENTICATE_MISSING_RIGHT = 4306
+    ENROLLMENT = 404
     CA = 503
+    CA_CSR_INVALID = 504
+    CA_CSR_PENDING = 505
     RESOURCE_NOT_FOUND = 601
     HSM = 707
     SELFSERVICE = 807
@@ -105,6 +109,11 @@ class SubscriptionError(privacyIDEAError):
         return ret
 
 
+class TokenImportException(privacyIDEAError):
+    def __init__(self, description, id=ERROR.IMPORTADMIN):
+        privacyIDEAError.__init__(self, description=description, id=id)
+
+
 class AuthError(privacyIDEAError):
     def __init__(self, description, id=ERROR.AUTHENTICATE, details=None):
         self.details = details
@@ -127,7 +136,12 @@ class ValidateError(privacyIDEAError):
 
 
 class RegistrationError(privacyIDEAError):
-    def __init__(self, description="registraion error!", id=ERROR.REGISTRATION):
+    def __init__(self, description="registration error!", id=ERROR.REGISTRATION):
+        privacyIDEAError.__init__(self, description=description, id=id)
+
+
+class EnrollmentError(privacyIDEAError):
+    def __init__(self, description="enrollment error!", id=ERROR.ENROLLMENT):
         privacyIDEAError.__init__(self, description=description, id=id)
 
 
@@ -146,6 +160,17 @@ class CAError(privacyIDEAError):
         privacyIDEAError.__init__(self, description=description, id=id)
 
 
+class CSRError(CAError):
+    def __init__(self, description="CSR invalid", id=ERROR.CA_CSR_INVALID):
+        privacyIDEAError.__init__(self, description=description, id=id)
+
+
+class CSRPending(CAError):
+    def __init__(self, description="CSR pending", id=ERROR.CA_CSR_PENDING, requestId=None):
+        privacyIDEAError.__init__(self, description=description, id=id)
+        self.requestId = requestId
+
+
 class UserError(privacyIDEAError):
     def __init__(self, description="user error!", id=ERROR.USER):
         privacyIDEAError.__init__(self, description=description, id=id)
