@@ -91,7 +91,7 @@ from .log import log_with
 from .config import (get_from_config, get_prepend_pin)
 from .user import (User,
                    get_username)
-from ..models import (TokenOwner, Challenge, cleanup_challenges)
+from ..models import (TokenOwner, TokenTokengroup, Tokengroup, Challenge, cleanup_challenges)
 from .challenge import get_challenges
 from privacyidea.lib.crypto import (encryptPassword, decryptPassword,
                                     generate_otpkey)
@@ -121,6 +121,10 @@ TWOSTEP_DEFAULT_DIFFICULTY = 10000
 log = logging.getLogger(__name__)
 
 
+class CHALLENGE_SESSION(object):
+    ENROLLMENT = "enrollment"
+
+
 class TOKENKIND(object):
     SOFTWARE = "software"
     HARDWARE = "hardware"
@@ -147,10 +151,14 @@ class CLIENTMODE(object):
     
 class ROLLOUTSTATE(object):
     CLIENTWAIT = 'clientwait'
+    # The rollout is pending in the backend, like CSRs that need to be approved
     PENDING = 'pending'
     # This means the user needs to authenticate to verify that the token was successfully enrolled.
     VERIFYPENDING = 'verify'
     ENROLLED = 'enrolled'
+    BROKEN = 'broken'
+    FAILED = 'failed'
+    DENIED = 'denied'
 
 
 class TokenClass(object):
@@ -164,6 +172,9 @@ class TokenClass(object):
     check_if_disabled = True
     # If the token provides means that the user has to prove/verify that the token was successfully enrolled.
     can_verify_enrollment = False
+    # If the token is enrollable via multichallenge
+    is_multichallenge_enrollable = False
+
 
     @log_with(log)
     def __init__(self, db_token):
@@ -241,6 +252,23 @@ class TokenClass(object):
         # set the tokenrealm
         self.set_realms([user.realm])
 
+    def add_tokengroup(self, tokengroup=None, tokengroup_id=None):
+        """
+        Adds a new tokengroup to this token.
+
+        :param tokengroup: The name of the token group to add
+        :type tokengroup: basestring
+        :param tokengroup_id: The id of the tokengroup to add
+        :type tokengroup_id: int
+        :return: True
+        """
+        if not tokengroup and not tokengroup_id:
+            raise ParameterError("You either need to specify a tokengroup name or id.")
+        r = TokenTokengroup(token_id=self.token.id,
+                            tokengroup_id=tokengroup_id,
+                            tokengroupname=tokengroup).save()
+        return r > 0
+
     @property
     def user(self):
         """
@@ -719,6 +747,17 @@ class TokenClass(object):
         tokenowner = self.token.first_owner
         return "" if not tokenowner else tokenowner.user_id
 
+    def set_tokengroups(self, tokengroups, add=False):
+        """
+        Set the list of the tokengroups of a token.
+
+        :param tokengroups: realms the token should be assigned to
+        :type tokengroups: list
+        :param add: if the tokengroups should be added and not replaced
+        :type add: boolean
+        """
+        self.token.set_tokengroups(tokengroups, add=add)
+
     def set_realms(self, realms, add=False):
         """
         Set the list of the realms of a token.
@@ -926,6 +965,19 @@ class TokenClass(object):
     def del_tokeninfo(self, key=None):
         self.token.del_info(key)
 
+    def del_tokengroup(self, tokengroup=None, tokengroup_id=None):
+        """
+        Removes a token group from a token.
+        You either need to specify the name or the ID of the tokengroup.
+
+        :param tokengroup: The name of the tokengroup
+        :type tokengroup: basestring
+        :param tokengroup_id: The ID of the tokengroup
+        :type tokengroup_id: int
+        :return: True in case of success
+        """
+        self.token.del_tokengroup(tokengroup=tokengroup, tokengroup_id=tokengroup_id)
+
     @check_token_locked
     def set_count_auth_success_max(self, count):
         """
@@ -1387,7 +1439,7 @@ class TokenClass(object):
         if otpkey is not None:
             response_detail["otpkey"] = {"description": "OTP seed",
                                          "value": "seed://{0!s}".format(otpkey),
-                                         "img": create_img(otpkey, width=200)}
+                                         "img": create_img(otpkey)}
 
         return response_detail
 
@@ -1537,16 +1589,22 @@ class TokenClass(object):
                     # Add the challenge to the options for check_otp
                     options["challenge"] = challengeobject.challenge
                     options["data"] = challengeobject.data
-                    # Now see if the OTP matches:
-                    otp_counter = self.check_otp(passw, options=options)
-                    if otp_counter >= 0:
-                        # We found the matching challenge, so lets return the
-                        #  successful result and delete the challenge object.
+                    if challengeobject.session == CHALLENGE_SESSION.ENROLLMENT:
+                        self.enroll_via_validate_2nd_step(passw, options=options)
                         challengeobject.delete()
-                        break
+                        # Basically we have a successfully answered challenge
+                        otp_counter = 0
                     else:
-                        # increase the received_count
-                        challengeobject.set_otp_status()
+                        # Now see if the OTP matches:
+                        otp_counter = self.check_otp(passw, options=options)
+                        if otp_counter >= 0:
+                            # We found the matching challenge, so lets return the
+                            # successful result and delete the challenge object.
+                            challengeobject.delete()
+                            break
+                        else:
+                            # increase the received_count
+                            challengeobject.set_otp_status()
 
         self.challenge_janitor()
         return otp_counter
@@ -1829,3 +1887,28 @@ class TokenClass(object):
         :return: True
         """
         return False
+
+    @classmethod
+    def enroll_via_validate(cls, g, content, user_obj):
+        """
+        This class method is used in the policy ENROLL_VIA_MULTICHALLENGE.
+        It enrolls a new token of this type and returns the necessary information
+        to the client by modifying the content.
+
+        :param g: context object
+        :param content: The content of a response
+        :param user_obj: A user object
+        :return: None, the content is modified
+        """
+        return True
+
+    def enroll_via_validate_2nd_step(self, passw, options=None):
+        """
+        This method is the optional second step of ENROLL_VIA_MULTICHALLENGE.
+        It is used in situations like the email token, sms token or push,
+        when enrollment via challenge response needs two steps.
+
+        :param options:
+        :return:
+        """
+        return True
