@@ -70,6 +70,7 @@ import traceback
 import datetime
 from privacyidea.lib.tokens.smstoken import HotpTokenClass
 from privacyidea.lib.tokens.hotptoken import VERIFY_ENROLLMENT_MESSAGE
+from privacyidea.lib.tokenclass import CHALLENGE_SESSION
 from privacyidea.lib.config import get_from_config
 from privacyidea.api.lib.utils import getParam
 from privacyidea.lib.utils import is_true, create_tag_dict
@@ -280,21 +281,27 @@ class EmailTokenClass(HotpTokenClass):
         if self.is_active() is True:
             counter = self.get_otp_count()
             log.debug("counter={0!r}".format(counter))
-            self.inc_otp_counter(counter, reset=False)
+
             # At this point we must not bail out in case of an
             # Gateway error, since checkPIN is successful. A bail
             # out would cancel the checking of the other tokens
             try:
-                message_template, mimetype = self._get_email_text_or_subject(options)
-                subject_template, _n = self._get_email_text_or_subject(options,
-                                                                   EMAILACTION.EMAILSUBJECT,
-                                                                   "Your OTP")
+                data = None
+                if options.get("session") != CHALLENGE_SESSION.ENROLLMENT:
+                    # Only if this is NOT an multichallenge enrollment, we try to send the email
+                    self.inc_otp_counter(counter, reset=False)
+                    message_template, mimetype = self._get_email_text_or_subject(options)
+                    subject_template, _n = self._get_email_text_or_subject(options,
+                                                                       EMAILACTION.EMAILSUBJECT,
+                                                                       "Your OTP")
+                    success, sent_message = self._compose_email(
+                        message=message_template,
+                        subject=subject_template,
+                        mimetype=mimetype)
 
                 # Create the challenge in the database
                 if is_true(get_from_config("email.concurrent_challenges")):
                     data = self.get_otp()[2]
-                else:
-                    data = None
                 db_challenge = Challenge(self.token.serial,
                                          transaction_id=transactionid,
                                          challenge=options.get("challenge"),
@@ -303,16 +310,11 @@ class EmailTokenClass(HotpTokenClass):
                                          validitytime=validity)
                 db_challenge.save()
                 transactionid = transactionid or db_challenge.transaction_id
-                # We send the email after creating the challenge for testing.
-                success, sent_message = self._compose_email(
-                    message=message_template,
-                    subject=subject_template,
-                    mimetype=mimetype)
 
             except Exception as e:
                 info = _("The PIN was correct, but the "
                          "EMail could not be sent!")
-                log.warning(info + " ({0!s})".format(e))
+                log.warning(info + " ({0!r})".format(e))
                 log.debug(u"{0!s}".format(traceback.format_exc()))
                 return_message = info
                 if is_true(options.get("exception")):
@@ -507,3 +509,57 @@ class EmailTokenClass(HotpTokenClass):
         """
         self.create_challenge()
         return {"message": VERIFY_ENROLLMENT_MESSAGE}
+
+    @classmethod
+    def enroll_via_validate(cls, g, content, user_obj):
+        """
+        This class method is used in the policy ENROLL_VIA_MULTICHALLENGE.
+        It enrolls a new token of this type and returns the necessary information
+        to the client by modifying the content.
+
+        :param g: context object
+        :param content: The content of a response
+        :param user_obj: A user object
+        :return: None, the content is modified
+        """
+        from privacyidea.lib.token import init_token
+        from privacyidea.lib.tokenclass import CLIENTMODE
+        token_obj = init_token({"type": cls.get_class_type(),
+                                "dynamic_email": 1}, user=user_obj)
+        content.get("result")["value"] = False
+        content.get("result")["authentication"] = "CHALLENGE"
+
+        detail = content.setdefault("detail", {})
+        # Create a challenge!
+        c = token_obj.create_challenge(options={"session": CHALLENGE_SESSION.ENROLLMENT})
+        # get details of token
+        init_details = token_obj.get_init_detail()
+        detail["transaction_ids"] = [c[2]]
+        chal = {"transaction_id": c[2],
+                "image": None,
+                "client_mode": CLIENTMODE.INTERACTIVE,
+                "serial": token_obj.token.serial,
+                "type": token_obj.type,
+                "message": _("Please enter your new email address!")}
+        detail["multi_challenge"] = [chal]
+        detail.update(chal)
+
+    def enroll_via_validate_2nd_step(self, passw, options=None):
+        """
+        This method is the optional second step of ENROLL_VIA_MULTICHALLENGE.
+        It is used in situations like the email token, sms token or push,
+        when enrollment via challenge response needs two steps.
+
+        The passw is entered during the first authentication step and it
+        contains the email address.
+
+        So we need to update the token with the email address and
+        we need to create a new challenge for the final authentication.
+
+        :param options:
+        :return:
+        """
+        self.del_tokeninfo("dynamic_email")
+        self.add_tokeninfo(self.EMAIL_ADDRESS_KEY, passw)
+        # Dynamically we remember that we need to do another challenge
+        self.currently_in_challenge = True
