@@ -11,16 +11,16 @@ import json
 from . import ldap3mock
 from .test_api_validate import LDAPDirectory
 from .base import MyApiTestCase
-from privacyidea.lib.error import (TokenAdminError, UserError)
 from privacyidea.lib.token import (get_tokens, remove_token, enable_token,
-                                   assign_token, unassign_token, init_token)
+                                   assign_token, init_token)
 from privacyidea.lib.user import User
 from privacyidea.lib.tokenclass import AUTH_DATE_FORMAT
 from privacyidea.lib.resolver import save_resolver
 from privacyidea.models import Token
 from privacyidea.lib.realm import (set_realm, delete_realm, set_default_realm)
 from privacyidea.api.lib.postpolicy import DEFAULT_POLICY_TEMPLATE_URL
-from privacyidea.lib.policy import ACTION, SCOPE, set_policy, delete_policy, LOGINMODE
+from privacyidea.lib.policy import (ACTION, SCOPE, set_policy, delete_policy,
+                                    LOGINMODE, ACTIONVALUE)
 
 
 PWFILE = "tests/testdata/passwords"
@@ -175,6 +175,11 @@ class APIAuthChallengeResponse(MyApiTestCase):
         from privacyidea.lib.token import set_pin
         set_pin("hotp1", "pin")
 
+    def tearDown(self):
+        remove_token('hotp1')
+        delete_policy('pol_cr')
+        delete_policy('webuilog')
+
     def test_01_challenge_response_at_webui(self):
         with self.app.test_request_context('/auth',
                                            method='POST',
@@ -199,6 +204,29 @@ class APIAuthChallengeResponse(MyApiTestCase):
             data = res.json
             self.assertEqual(data.get("result").get("value").get("username"), "selfservice")
 
+    def test_02_auth_chal_resp_multi_token(self):
+        # create another token
+        Token("hotp2", "hotp", otpkey=self.otpkey, userid=1004,
+              resolver=self.resolvername1, realm=self.realm1).save()
+        set_policy(name="pol_otppin", scope=SCOPE.AUTH,
+                   action="{0!s}={1!s}".format(ACTION.OTPPIN, ACTIONVALUE.USERSTORE))
+        with self.app.test_request_context('/auth',
+                                           method='POST',
+                                           data={"username": "selfservice",
+                                                 "password": 'test'}):
+            res = self.app.full_dispatch_request()
+            self.assertEqual(res.status_code, 401, res)
+            result = res.json.get('detail')
+            self.assertTrue('multi_challenge' in result, result)
+            self.assertEqual({'hotp1', 'hotp2'}, set([x['serial'] for x in result.get('multi_challenge')]))
+
+        # check if we have both serials in the audit log
+        ae = self.find_most_recent_audit_entry(action='POST /auth')
+        self.assertTrue({"hotp1", "hotp2"}.issubset(set(ae.get('serial').split(','))), ae)
+
+        delete_policy('pol_otppin')
+        remove_token('hotp2')
+
 
 class APISelfserviceTestCase(MyApiTestCase):
 
@@ -1006,6 +1034,8 @@ class APISelfserviceTestCase(MyApiTestCase):
             ACTION.USERPAGESIZE, 20))
         set_policy(name="webui3", scope=SCOPE.WEBUI, action="{0!s}={1!s}".format(
             ACTION.LOGOUTTIME, 200))
+        set_policy(name="webui4", scope=SCOPE.WEBUI, action="{0!s}={1!s}".format(
+            ACTION.AUDITPAGESIZE, 20))
         with self.app.test_request_context('/auth',
                                            method='POST',
                                            data={"username": "selfservice@realm1",
@@ -1016,6 +1046,7 @@ class APISelfserviceTestCase(MyApiTestCase):
             self.assertTrue(result.get("status"), res.data)
             # Test logout time
             self.assertEqual(result.get("value").get("logout_time"), 200)
+            self.assertEqual(result.get("value").get("audit_page_size"), 20)
             self.assertEqual(result.get("value").get("token_page_size"), 20)
             self.assertEqual(result.get("value").get("user_page_size"), 20)
             self.assertEqual(result.get("value").get("policy_template_url"),
@@ -1389,6 +1420,34 @@ class PolicyConditionsTestCase(MyApiTestCase):
             # only HOTP tokens
             self.assertEqual(set(result['value'].keys()), {'hotp'})
 
+        # We set a new policy condition for users (bob), to be allowed to enroll TOTP
+        # tokens only in special cases. This condition will not match for certain tokens.
+        with self.app.test_request_context('/policy/totp_enroll',
+                                           json={'action': "enrollTOTP",
+                                                 'scope': SCOPE.USER,
+                                                 'realm': '',
+                                                 'active': True,
+                                                 'conditions': [
+                                                     ["userinfo", "username", "equals", "bob", True],
+                                                     ["token", "count", "equals", "1", True],
+                                                 ]
+                                                 },
+                                           method='POST',
+                                           headers={'Authorization': self.at}):
+            res = self.app.full_dispatch_request()
+            self.assertEqual(res.status_code, 200)
+
+        # check the rights of bob. The TOTP enroll right will basically be there
+        with self.app.test_request_context('/auth/rights',
+                                           method='GET',
+                                           headers={'Authorization': bob_token}):
+            res = self.app.full_dispatch_request()
+            self.assertEqual(res.status_code, 200)
+            result = res.json.get("result")
+            # HOTP and TOTP tokens
+            self.assertIn('hotp', set(result['value'].keys()))
+            self.assertIn('totp', set(result['value'].keys()))
+
         # check the rights of manager
         with self.app.test_request_context('/auth/rights',
                                            method='GET',
@@ -1412,11 +1471,11 @@ class PolicyConditionsTestCase(MyApiTestCase):
         status_code, result = _try_enroll(bob_token, 'hotp')
         self.assertEqual(status_code, 200)
         self.assertEqual(result, {"status": True, "value": True})
-        # ... but not TOTP ...
+        # ... but not TOTP, since the extended condition does not match
         status_code, result = _try_enroll(bob_token, 'totp')
         self.assertEqual(status_code, 403)
         self.assertFalse(result['status'])
-        self.assertIn("not allowed to enroll this token type", result['error']['message'])
+        self.assertIn("has conditions on tokens, but a token object is not available", result['error']['message'])
         # ... and certainly not SPASS
         status_code, result = _try_enroll(bob_token, 'spass')
         self.assertEqual(status_code, 403)
