@@ -723,6 +723,64 @@ class UserNotificationTestCase(MyTestCase):
                                    "emailconfig": "myserver",
                                    "To": NOTIFY_TYPE.EMAIL,
                                    "To " + NOTIFY_TYPE.EMAIL:
+                                       "recp@example.com",
+                                   "reply_to": NOTIFY_TYPE.EMAIL,
+                                   "reply_to" + NOTIFY_TYPE.EMAIL:
+                                       "recp@example.com"}}}
+
+        un_handler = UserNotificationEventHandler()
+        res = un_handler.do("sendmail", options=options)
+        self.assertTrue(res)
+
+    @smtpmock.activate
+    def test_12_send_to_tokenowner(self):
+        # setup realms
+        self.setUp_user_realms()
+
+        r = add_smtpserver(identifier="myserver", server="1.2.3.4", tls=False)
+        self.assertTrue(r > 0)
+
+        smtpmock.setdata(response={"recp@example.com": (200, "OK")},
+                         support_tls=False)
+
+        g = FakeFlaskG()
+        audit_object = FakeAudit()
+        audit_object.audit_data["serial"] = "123456"
+
+        g.logged_in_user = {"username": "admin",
+                            "role": "admin",
+                            "realm": ""}
+        g.audit_object = audit_object
+
+        builder = EnvironBuilder(method='POST',
+                                 data={'serial': "OATH123456"},
+                                 headers={})
+
+        env = builder.get_environ()
+        # Set the remote address so that we can filter for it
+        env["REMOTE_ADDR"] = "10.0.0.1"
+        g.client_ip = env["REMOTE_ADDR"]
+        req = Request(env)
+        req.all_data = {"serial": "SomeSerial",
+                        "user": "cornelius"}
+        req.User = User("cornelius", self.realm1)
+        resp = Response()
+        resp.data = """{"result": {"value": true},
+        "detail": {"registrationcode": "12345678910"}
+        }
+        """
+        options = {"g": g,
+                   "request": req,
+                   "response": resp,
+                   "handler_def": {
+                       "conditions": {"serial": "123.*"},
+                       "options": {"body": "your {registrationcode}",
+                                   "emailconfig": "myserver",
+                                   "To": NOTIFY_TYPE.TOKENOWNER,
+                                   "To " + NOTIFY_TYPE.TOKENOWNER:
+                                       "recp@example.com",
+                                   "reply_to": NOTIFY_TYPE.TOKENOWNER,
+                                   "reply_to" + NOTIFY_TYPE.TOKENOWNER:
                                        "recp@example.com"}}}
 
         un_handler = UserNotificationEventHandler()
@@ -777,6 +835,9 @@ class UserNotificationTestCase(MyTestCase):
                                    "emailconfig": "myserver",
                                    "To": NOTIFY_TYPE.INTERNAL_ADMIN,
                                    "To " + NOTIFY_TYPE.INTERNAL_ADMIN:
+                                       "super",
+                                   "reply_to": NOTIFY_TYPE.INTERNAL_ADMIN,
+                                   "reply_to" + NOTIFY_TYPE.INTERNAL_ADMIN:
                                        "super"}}}
 
         un_handler = UserNotificationEventHandler()
@@ -793,6 +854,9 @@ class UserNotificationTestCase(MyTestCase):
                                    "emailconfig": "myserver",
                                    "To": NOTIFY_TYPE.INTERNAL_ADMIN,
                                    "To " + NOTIFY_TYPE.INTERNAL_ADMIN:
+                                       "testadmin",
+                                   "reply_to": NOTIFY_TYPE.INTERNAL_ADMIN,
+                                   "reply_to" + NOTIFY_TYPE.INTERNAL_ADMIN:
                                        "testadmin"}}}
 
         un_handler = UserNotificationEventHandler()
@@ -845,6 +909,7 @@ class UserNotificationTestCase(MyTestCase):
                        "conditions": {"serial": "123.*"},
                        "options": {"body": "your {registrationcode}",
                                    "emailconfig": "myserver",
+                                   "reply_to": NOTIFY_TYPE.LOGGED_IN_USER,
                                    "To": NOTIFY_TYPE.LOGGED_IN_USER}}}
 
         un_handler = UserNotificationEventHandler()
@@ -862,6 +927,7 @@ class UserNotificationTestCase(MyTestCase):
                        "conditions": {"serial": "123.*"},
                        "options": {"body": "your {registrationcode}",
                                    "emailconfig": "myserver",
+                                   "reply_to": NOTIFY_TYPE.LOGGED_IN_USER,
                                    "To": NOTIFY_TYPE.LOGGED_IN_USER}}}
         un_handler = UserNotificationEventHandler()
         res = un_handler.do("sendmail", options=options)
@@ -917,6 +983,9 @@ class UserNotificationTestCase(MyTestCase):
                                    "emailconfig": "myserver",
                                    "To": NOTIFY_TYPE.ADMIN_REALM,
                                    "To " + NOTIFY_TYPE.ADMIN_REALM:
+                                       "realm1",
+                                   "reply_to": NOTIFY_TYPE.ADMIN_REALM,
+                                   "reply_to" + NOTIFY_TYPE.ADMIN_REALM:
                                        "realm1"}}}
         un_handler = UserNotificationEventHandler()
         res = un_handler.do("sendmail", options=options)
@@ -1475,6 +1544,7 @@ class UserNotificationTestCase(MyTestCase):
         os.chmod("tests/testdata/testOATH123456.txt", 0o400)
         un_handler = UserNotificationEventHandler()
         # Check that an error is written to the logfile
+        # Note: this does not work when tests are run as root
         with mock.patch("logging.Logger.error") as mock_log:
             un_handler.do("savefile", options=options)
             call_args = mock_log.call_args
