@@ -58,8 +58,7 @@ class LibPolicyTestCase(MyTestCase):
         return pin == "FAKE"
 
     @staticmethod
-    def fake_check_token_list(tokenobject_list, passw, user=None, options=None, allow_reset_all_tokens=True,
-                              result=False):
+    def fake_check_token_list(tokenobject_list, passw, user=None, options=None, allow_reset_all_tokens=True, result=False):
         return result, "some text"
 
     def test_01_otppin(self):
@@ -926,4 +925,4 @@ class LibPolicyTestCase(MyTestCase):
         remove_token("TOKFAIL")
         remove_token("TOKMATCH")
         delete_policy("pol1")
-        delete_policy("pol2")
\ No newline at end of file
+        delete_policy("pol2")
