@@ -96,7 +96,7 @@ class SMTPServerTestCase(MyTestCase):
         self.assertRaises(ResourceNotFoundError, get_smtpserver, "notExisting")
 
     @smtpmock.activate
-    def test_05_test_email(self):
+    def test_05_test_email_smtp(self):
         smtpmock.setdata(response={"recp@example.com": (200, "OK")},
                          support_tls=True)
         identifier = "newConfig"
@@ -104,7 +104,7 @@ class SMTPServerTestCase(MyTestCase):
         port = 25
         username = "mailsender"
         password = "secret"
-        sender = "mailsender@exmaple.com"
+        sender = "mailsender@example.com"
         tls = True
         recipient = "user@example.com"
 
@@ -130,6 +130,46 @@ class SMTPServerTestCase(MyTestCase):
         self.assertEqual(parsed_email.get_content_type(), 'image/png', parsed_email)
         self.assertEqual(parsed_email.get('To'), recipient, parsed_email)
         self.assertEqual(parsed_email.get('Subject'), "Test Email with image", parsed_email)
+        # Check, that the mock SMTP server actually has NOT been configured as SMTP_SSL
+        self.assertFalse(smtpmock.get_smtp_ssl())
+
+    @smtpmock.activate
+    def test_06_test_email_smtp_ssl(self):
+        smtpmock.setdata(response={"recp@example.com": (200, "OK")},
+                         support_tls=False)
+        identifier = "newConfig"
+        server = "smtps://mailserver"
+        port = 25
+        username = "mailsender"
+        password = "secret"
+        sender = "mailsender@example.com"
+        tls = False
+        recipient = "user@example.com"
+
+        s = dict(identifier=identifier, server=server, port=port,
+                 username=username, password=password, sender=sender,
+                 tls=tls)
+        r = SMTPServer.test_email(s, recipient,
+                                  "Test Email from privacyIDEA",
+                                  "This is a test email from privacyIDEA. "
+                                  "The configuration %s is working." % identifier)
+        self.assertTrue(r)
+        parsed_email = email.message_from_string(smtpmock.get_sent_message())
+        self.assertEqual(parsed_email.get_content_type(), 'text/plain', parsed_email)
+        self.assertEqual(parsed_email.get('To'), recipient, parsed_email)
+        self.assertEqual(parsed_email.get('Subject'), "Test Email from privacyIDEA", parsed_email)
+
+        # Now with an already prepared MIME email
+        msg = MIMEImage(binascii.a2b_base64(PNG_IMG))
+        r = SMTPServer.test_email(s, recipient, "Test Email with image",
+                                  msg)
+        self.assertTrue(r)
+        parsed_email = email.message_from_string(smtpmock.get_sent_message())
+        self.assertEqual(parsed_email.get_content_type(), 'image/png', parsed_email)
+        self.assertEqual(parsed_email.get('To'), recipient, parsed_email)
+        self.assertEqual(parsed_email.get('Subject'), "Test Email with image", parsed_email)
+        # Check, if the mock SMTP server actually has been configured as SMTP_SSL
+        self.assertTrue(smtpmock.get_smtp_ssl())
 
 
 class SMTPServerQueueTestCase(MockQueueTestCase):
