@@ -29,7 +29,7 @@ import hashlib
 import binascii
 import warnings
 from privacyidea.lib.token import (create_tokenclass_object,
-                                   get_tokens,
+                                   get_tokens, list_tokengroups,
                                    get_token_type, check_serial,
                                    get_num_tokens_in_realm,
                                    get_realms_of_token,
@@ -56,8 +56,8 @@ from privacyidea.lib.token import (create_tokenclass_object,
                                    set_validity_period_end,
                                    set_validity_period_start, remove_token, delete_tokeninfo,
                                    import_token, get_one_token, get_tokens_from_serial_or_user,
-                                   get_tokens_paginated_generator)
-
+                                   get_tokens_paginated_generator, assign_tokengroup, unassign_tokengroup)
+from privacyidea.lib.tokengroup import set_tokengroup, delete_tokengroup
 from privacyidea.lib.error import (TokenAdminError, ParameterError,
                                    privacyIDEAError, ResourceNotFoundError)
 from privacyidea.lib.tokenclass import DATE_FORMAT
@@ -1619,24 +1619,24 @@ class TokenOutOfBandTestCase(MyTestCase):
         self.assertEqual(r, 0)
 
         r, r_dict = check_token_list([token1], pin1, user=user, options={})
-        self.assertFalse(r)
+        self.assertFalse(r, r_dict)
         transaction_id = r_dict.get("transaction_id")
 
         # Now we check the status of the challenge several times and verify that the
         # failcounter is not increased:
         for i in range(1, 10):
             r, r_dict = check_token_list([token1], "", user=user, options={"transaction_id": transaction_id})
-            self.assertFalse(r)
-            self.assertEqual(r_dict.get("type"), "tiqr")
+            self.assertFalse(r, r_dict)
+            self.assertEqual(r_dict.get("type"), "tiqr", r_dict)
 
         r = token1.get_failcount()
         self.assertEqual(r, 0)
 
         # Now set the challenge to be answered and recheck:
-        Challenge.query.filter(Challenge.transaction_id == transaction_id).update({"otp_valid": 1})
+        Challenge.query.filter(Challenge.transaction_id == transaction_id).update({"otp_valid": True})
         r, r_dict = check_token_list([token1], "", user=user, options={"transaction_id": transaction_id})
-        self.assertTrue(r)
-        self.assertEqual(r_dict.get("message"), "Found matching challenge")
+        self.assertTrue(r, r_dict)
+        self.assertEqual(r_dict.get("message"), "Found matching challenge", r_dict)
 
         remove_token(pin1)
 
@@ -2054,3 +2054,57 @@ class PINChangeTestCase(MyTestCase):
                          "g": g})
 
         delete_policy("minpin")
+
+
+class TokenGroupTestCase(MyTestCase):
+
+    def test_01_add_tokengroups(self):
+        # Create tokens
+        serials = ["s1", "s2"]
+        for s in serials:
+            tok = init_token({"serial": s, "type": "spass"})
+
+        # create tokengroups
+        groups = [("g1", "Test A"), ("g2", "test B")]
+        for g in groups:
+            set_tokengroup(g[0], g[1])
+
+        assign_tokengroup("s1", "g1")
+        assign_tokengroup("s1", "g2")
+        assign_tokengroup("s2", "g2")
+
+        # Check the tokengroups of the first token
+        tok1 = get_one_token(serial="s1")
+        self.assertEqual(tok1.token.tokengroup_list[0].tokengroup.name, "g1")
+        self.assertEqual(tok1.token.tokengroup_list[1].tokengroup.name, "g2")
+
+        # check the tokengroups of the 2nd token
+        tok2 = get_one_token(serial="s2")
+        self.assertEqual(tok2.token.tokengroup_list[0].tokengroup.name, "g2")
+
+        # Test a missing group information
+        self.assertRaises(ResourceNotFoundError, assign_tokengroup, "s1")
+
+        # list tokengroup assignments
+        grouplist = list_tokengroups()
+        self.assertEqual(len(grouplist), 3)
+        # one token in group1
+        grouplist = list_tokengroups("g1")
+        self.assertEqual(len(grouplist), 1)
+        self.assertEqual(grouplist[0].token.serial, "s1")
+        # two tokens in group2
+        grouplist = list_tokengroups("g2")
+        self.assertEqual(len(grouplist), 2)
+
+        # unassign tokengroups
+        r = tok1.del_tokengroup("g1")
+        # only the 2nd group remains
+        self.assertEqual(tok1.token.tokengroup_list[0].tokengroup.name, "g2")
+        # remove it
+        unassign_tokengroup("s2", "g2")
+        tok2 = get_one_token(serial="s2")
+        self.assertEqual(len(tok2.token.tokengroup_list), 0)
+
+        # cleanup
+        for s in serials:
+            remove_token(s)
